/// <reference path="../common/js/jquery-1.8.1.min.js" />

/// <reference path="../common/js/jquery-ui-1.8.23.custom.min.js" />

/// <reference path="../common/js/common.js" />

/* rect object */
var Rect = function(x, y, w, h) {
	this.left = Math.floor(x);
	this.top = Math.floor(y);
	this.right  = Math.floor(x + w);
	this.bottom = Math.floor(y + h);
	this.width = w;
	this.height = h;
};
Rect.prototype.add = function(x, y) {
	this.left = Math.floor(this.left + x);
	this.right = Math.floor(this.right + x);
	this.top = Math.floor(this.top + y);
	this.bottom = Math.floor(this.bottom + y);
	this.width = this.right - this.left;
	this.height = this.bottom - this.top;
};
Rect.prototype.center = function() {
	var x = (this.left + this.right) / 2;
	var y = (this.top + this.bottom) / 2;
	return new Point(Math.floor(x), Math.floor(y));
};


/**
 * CurrentPage Class Definition 
 */
/* constructor */
var ContentPage = function(pageNum, pageImageUrl) {
	
	this.pageNumber = 0;
	this.pageImageUrl = pageImageUrl;
	this.bInitialized = false;
	
	/* init screen image */
	this.image = null;
	
	/* array page objects */
	this.pageObjects = new Array();
};

/* Load page */
ContentPage.prototype.load = function(onCompletedFunc) {
	
	/* load image  */
	this.image = new Image();	
	this.image.onerror = function() {
		//alert(this.pageImageUrl + " hasn't loaded..");
	};
	this.image.onload = function(context) {
		/* load complete */
		if(onCompletedFunc) {		
			onCompletedFunc();
		}
	}
	
	this.image.src = this.pageImageUrl;
};

/* add 1 page to pageObject */
ContentPage.prototype.addPageObject = function(pageObject) {
	this.pageObjects.push(pageObject);
};

/* init Json object using for init pages */
ContentPage.prototype.addPageObjects = function(pageObjects) {
	for(var i = 0; i < pageObjects.length; i++) {
		var obj = pageObjects[i];
	
		var pageObj = null;
		switch(obj.mediaType) {
			case 1:
				//mediaType = 1
				if(obj.actionType == 0){													
					pageObj = new imageNoAction(     /* no action */           
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible    		
								);
				}else if(obj.actionType == 1){	/* play video */												
					pageObj = new videoType1(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,
									obj.mediaFile,
									obj.media    		
								);
				}else if(obj.actionType == 2){														
					pageObj = new audioType1(    /* play audio */    						        
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,
									obj.audioFile    		
								);
				}else if(obj.actionType == 3){ /*open new url */													
					pageObj = new linkURL(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.action,
									obj.linkUrl,
									obj.browserType	       					
								);
				}else if(obj.actionType == 4 ){ /*image preview */	    
					pageObj = new imagePreview(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.imagePreview       					
								);
				}else if(obj.actionType == 5 ){  /*jump page */	   
					pageObj = new jumpPage(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.jumpPage       					
								);
				}else if(obj.actionType == 6 ){ /*send mail */	    
					pageObj = new sendMail(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.emailSubject,
									obj.emailAddress       					
								);
				}else if(obj.actionType == 8){ /* show popup */	    
					pageObj = new openPopUp(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.content      					
								);
				}else if(obj.actionType == 10 ){ /* move to another content */						   
					pageObj = new moveToContent(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.contentId,
									obj.pageNo       					
								);
				}
				             
			break;
			case 2:
				pageObj = new  videoType2(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.mediaFile,
									obj.media,
									obj.embed      					
								);
			break;
			case 3:
				pageObj = new  audioType3(                
									obj.mediaType,     
									obj.actionType,    
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.audioFile,
									obj.playType       					
								);
			break;
			case 4:
				// mediaType = 4
				objType4_5.push(obj);	
				pageObj = new  listImage(                
									obj.mediaType,     									
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.dataObjects,
									obj.triggerType       	     					
								);
			break;
			case 5:
				// mediaType = 5	
				objType4_5.push(obj);
				pageObj = new  listVideo(                
									obj.mediaType,     									
									obj.id,            
									obj.imageUrl,      	
									obj.x,             
									obj.y,             
									obj.width,         
									obj.height,        
									obj.visible,       
									obj.dataObjects												
								);
			break;
			case 6:
				// mediaType = 6
				pageObj = new trigger(
										obj.mediaType,
										obj.actionType,
										obj.id,
										obj.imageUrl,
										obj.x,
										obj.y,
										obj.width,
										obj.height,
										obj.index,
										obj.target,
										obj.visible,
										obj.action					
									);
			break;
			case 7:
				// mediaType = 7
				pageObj = new text(
										obj.mediaType,
										obj.actionType,
										obj.id,
										obj.x,
										obj.y,
										obj.width,
										obj.height,
										obj.visible,
										obj.action,
										obj.content				
									);
			break;
			default:
				// default Object
				pageObj = new PageObject();
				pageObj.setup(
							obj.mediaType,
							obj.actionType,
							obj.id,
							obj.imageUrl,
							obj.x,
							obj.y,
							obj.width,
							obj.height,
							obj.visible,
							obj.action					
						);
		}
		// add pageObject
		this.addPageObject(pageObj);
	}
};

/* drawing for current page */
ContentPage.prototype.drawPage = function(context) {
	/*set status is loading page */
	isLoadingObject = true;
	/* lock all controls in page */
	disableAllControl();
	/* init array image */
	arrImage = [];
	nCountObjectLoad = 0;
	$('#divImageLoading').css('display','block');	
    	
	var width = this.image.width;
	var height = this.image.height;
	
	/* set width canvas */
	if(width > height){
		if(widthContentImage > heightContentImage){
			widthEachPage = widthContentImage;
			heightEachPage = heightContentImage;
		}else{
			widthEachPage = heightContentImage;
			heightEachPage = widthContentImage;
		}
	}else{
		if(widthContentImage < heightContentImage){
			widthEachPage = widthContentImage;
			heightEachPage = heightContentImage;
		}else{
			widthEachPage = heightContentImage;
			heightEachPage = widthContentImage;
		}
	}
	
	$("#offscreen").attr('height', heightEachPage);
	$("#offscreen").attr('width', widthEachPage);
	
	var objPageTemp = this.pageObjects;
	var img = new Image();
	img.onload = function(){		
		context.drawImage(img, 0, 0, widthEachPage, heightEachPage);
		flip();
		
		/* draw linkList */
   		drawGetWebContent4(context);
		
		if(objPageTemp.length > 0){						
			// draw object
			for(var i = 0; i < objPageTemp.length; i++) {				
				if(objPageTemp[i].imageUrl != null){			
					objPageTemp[i].drawPageObject(context);	
				}else{
					nCountObjectLoad++;
					
					if(nCountObjectLoad == pageObjects.length){
						/* draw marking */
						drawMarkingOnScreen();	
						
						/*set status is loading page */
						isLoadingObject = false;
						/* enable controls in page */
						if(ClientData.IsAddingMarking() == true){		
							disableAllControl();	
							enableButtonMarking()					
						}else{
							enableAllControl();
						}
						/*finish loading */
						$('#divImageLoading').css('display','none');
						
						/* set pos video */
						zoomVideo();
					}
				}
			}	
		}else{
			/* draw marking */
			drawMarkingOnScreen();	
			
			/*set status is loading page */
			isLoadingObject = false;
			/* enable controls in page */
			if(ClientData.IsAddingMarking() == true){		
				disableAllControl();	
				enableButtonMarking()					
			}else{
				enableAllControl();
			}
			
			/*finish loading */
			$('#divImageLoading').css('display','none');
			
			/* set pos video */
			zoomVideo();
		}
	}	
	img.src = this.image.src;			
};

/* If have hitTest on the objects within the page and return PageObject */
ContentPage.prototype.hitTest = function(px, py) {
	for(var i = 0; i < this.pageObjects.length; i++) {
		if(this.pageObjects[i].hitTest(px, py)) {
			return this.pageObjects[i];
		}
	}
	return null;
};

/* search by Object ID */
ContentPage.prototype.findById = function(id) {
	for(var i = 0; i < this.pageObjects.length; i++) {
		if(this.pageObjects[i].id == id) {
			return this.pageObjects[i];
		}
	}
	return null;	
};


/**
 * PageObject Class Definition 
 */

/* constructor */
var PageObject = function(id, imageUrl, x, y, w, h, visible, actionFunction,displayBorder) {
};

/* setup each properties */
PageObject.prototype.setup = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, actionFunction,displayBorder) {
	
	this.mediaType = mediaType;
	this.actionType = actionType;
	this.id = id;
	this.x = x;
	this.y = y;
	this.width = w;
	this.height = h;
	this.displayBorder = displayBorder;
	this.visible = visible;
	this.imageUrl = imageUrl;
	this.image = new Image();

	this.image.src = this.imageUrl;	
};

/* prototype hitTest */
PageObject.prototype.hitTest = function(px, py) {	
	if(this.x <= px && px <= (this.x + this.width)) {			
		if(this.y <= py && py <= (this.y + this.height)) {			
			return true;
		}
	}
	return false;
};
var check;
/* draw page Object */
PageObject.prototype.drawPageObject = function(context) {
	if(this.visible) {
		/* init data */	
		var mediaType = this.mediaType;	
		var displayBorder = this.displayBorder;
		var x = this.x;
		var y = this.y;
		var w = this.width;
		var h = this.height;		
		
		if(mediaType == '5'){		
			nCountObjectLoad ++;	
			context.strokeStyle = "black";
			context.lineWidth = "1";
			context.strokeRect(x, y, w, h);
		}
		
		if(this.imageUrl != ''){		
			/* init image  */	
			var img = new Image();
			
			img.onload = function(){
				nCountObjectLoad ++;
				
				/* create object store value image */
				var objTemp = [];
				objTemp.mediaType = mediaType;
				objTemp.img = img;
				objTemp.x = x;
				objTemp.y = y;
				objTemp.w = w;
				objTemp.h = h;
				objTemp.displayBorder = displayBorder;
				
				/* push to array */
				arrImage.push(objTemp)
							
				if(mediaType == '2'){/*  media */
					img.width = 50;
					img.height = 50;
					
					if(img.width > w){
						context.drawImage(img, x, y, w, h);
					}else{	
						/*draw image and border */				
						context.drawImage(img, x +((w - img.width)/2) , y + ((h- img.height)/2), img.width, img.height);			
						if(displayBorder == '1'){/* draw border */
							context.strokeStyle = "black";
							context.lineWidth = "1";
							context.strokeRect(x, y, w, h);
						}						
					}
					
				}else if(mediaType == '3'){/* audio */
					img.width = 50;
					img.height = 50;
					context.drawImage(img, x, y, img.width, img.height);
				}else{					
					context.drawImage(img, x, y, w, h);
				}
				flip();

			
				
				if(nCountObjectLoad == pageObjects.length){	
	
					/* draw image */
					// for(var nIndex = 0; nIndex < arrImage.length; nIndex ++){
						// if(arrImage[nIndex].mediaType == '2'){/*  media */
// 					
							// if(arrImage[nIndex].img.width > arrImage[nIndex].w){
								// context.drawImage(arrImage[nIndex].img, arrImage[nIndex].x, arrImage[nIndex].y, arrImage[nIndex].w, arrImage[nIndex].h);
							// }else{	
								// /*draw image and border */				
								// context.drawImage(arrImage[nIndex].img, arrImage[nIndex].x +((arrImage[nIndex].w - arrImage[nIndex].img.width)/2) , arrImage[nIndex].y + ((arrImage[nIndex].h- arrImage[nIndex].img.height)/2), arrImage[nIndex].img.width, arrImage[nIndex].img.height);			
								// if(arrImage[nIndex].displayBorder == '1'){/* draw border */
									// context.strokeStyle = "black";
									// context.lineWidth = "1";
									// context.strokeRect(arrImage[nIndex].x, arrImage[nIndex].y, arrImage[nIndex].w, arrImage[nIndex].h);
								// }	
							// }
// 							
						// }else if(arrImage[nIndex].mediaType == '3'){/* audio */
							// context.drawImage(arrImage[nIndex].img, arrImage[nIndex].x, arrImage[nIndex].y, arrImage[nIndex].img.width, arrImage[nIndex].img.height);
						// }else{					
							// context.drawImage(arrImage[nIndex].img, arrImage[nIndex].x, arrImage[nIndex].y, arrImage[nIndex].w, arrImage[nIndex].h);
						// }	
// 								
					// }			

					
					// flip();		
									
					/* draw marking */
	   				drawMarkingOnScreen();	
	   				
					/*set status is loading page */
					isLoadingObject = false;
										
					/* enable controls in page */
					if(ClientData.IsAddingMarking() == true){		
						disableAllControl();	
						enableButtonMarking()					
					}else{
						enableAllControl();
					}
					
					/*finish loading */
					$('#divImageLoading').css('display','none');	
					
					/* set pos video */
					zoomVideo();
				}									
			};
			img.src = this.imageUrl;
		}else{
			if(nCountObjectLoad == pageObjects.length){						
				/* draw marking */
   				drawMarkingOnScreen();	
   				
				/*set status is loading page */
				isLoadingObject = false;
				/* enable controls in page */				
				if(ClientData.IsAddingMarking() == true){		
					disableAllControl();	
					enableButtonMarking()					
				}else{
					enableAllControl();
				}
				
				/*finish loading */
				$('#divImageLoading').css('display','none');	
				
				/* set pos video */
				zoomVideo();
			}
		}										
	}		 
};

/**
 * Content Class Definition 
 */
var Content = function() {
	
	this.pageCount = 0;	
	this.pageIndex = 0;
	this.pageImages = null;
	this.pageObjects = null;
	this.currentPage = null;

};

/* ste page images (json data) */
Content.prototype.setPageImages = function(pageCount, imageUrls) {
	this.pageCount = pageCount;
	this.pageImages = imageUrls;
	return this;
};

/* set page objects data (json data) */
Content.prototype.setPageObjects = function(pageObjects) {
	this.pageObjects = pageObjects;
	return this;
};

/* Set up page */
Content.prototype.setUpPage = function(pageIndex) {		
 	this.currentPage = createPage(
 							this.pageIndex, 
							this.pageImages,
							this.pageObjects,
							function() { 
								adjustPage();
							});						
	return this;
};

/* check next page exists */
Content.prototype.hasNextPage = function() {
	
	if((this.pageIndex + 1) > (this.pageCount - 1)) {
		return false;
	}
	return true;
};

/* setup next page */
Content.prototype.nextPage = function() {
	this.pageIndex++;
	if(this.pageIndex > (this.pageCount - 1)) {
		this.pageIndex = this.pageCount - 1;
	}
	this.setUpPage(this.pageIndex);
	return this;
};

/* check previous page exists */
Content.prototype.hasPreviousPage = function() {
	if((this.pageIndex - 1) < 0) {
		return false;
	}
	return true;
};

/* setup previous page */
Content.prototype.previousPage = function() {
	this.pageIndex--;
	if(this.pageIndex < 0) {
		this.pageIndex = 0;
	}
	this.setUpPage(this.pageIndex);	
	return this;
};

/* setup previous page */
Content.prototype.toPage = function(index) {
	this.pageIndex = eval(index);
	if(this.pageIndex < 0) {
		this.pageIndex = 0;
	}
	this.setUpPage(this.pageIndex);	
	return this;
};

/* check status button move page when change page */
function checkExistNextPrePage(){
	//check next page
	if(getContent().hasNextPage()){
		//$('#nextpage').show();
		enable('#nextpage');
	}else{
		//$('#nextpage').hide();
		disable('#nextpage');	
		$('#button_next_canvas').css('opacity','0');	
	}
	
	//check prev page
	if(getContent().hasPreviousPage()){
		//$('#prevpage').show();
		enable('#prevpage');
	}else{
		//$('#prevpage').hide();
		disable('#prevpage');
		$('#button_pre_canvas').css('opacity','0');
		
	}
	
}

/* init variable */
var content = new Content();
var srcRect = new Rect(0, 0, 0, 0);
var destRect = new Rect(0, 0, 0, 0);
var isDestPositionDetect = false;
var destPosX = 0, destPosY = 0;
var	 moveX = 0, moveY = 0;
var userScale = 1;
var scaleDelta = 0.5;
var scaleX, scaleY;
var canvasObject;
var contextObject;
var objType4_5 = [];
/*var objType5;
var isObjType4 = false;
var isObjType5 = false;*/	
/* variable for event mouse */

var moveFlag = false;
var cancelClick = false;
var timeCreateMemo;

var imageData;
var isLoadBackgroundComplete = false;


/* Point objects */
var Point = function(x, y) {
	this.x = x;
	this.y = y;
};

/**
 * constants 
 */
var marginX = 13;
var marginY = 65;



/**
 * Transition Class Definition 
 */
var Transition = function() {};
/* Flip Next Page Transition */
Transition.prototype.flipNextPage = function() {
	var w = $('#main').attr('width');
	removeObject();
	
	if(getContent().hasNextPage()) {						
		$('#main').animate(
				{left : '-' + w + 'px'}, 
				{ 
					duration : "slow",
					complete : function() {
			   
						// change page
						$('#main').css('left', w + 'px');												
						// animate 
						$('#main').animate(
							{left : marginX + 'px'}, 
							{duration : 'slow',
							complete: function(){
								isFirstLoad = true;
								screenFit(); 
							}
						});	
							
						 //change pageIndex and image bookmark						
						getContent().nextPage();
						initImageBookmark();
						checkExistNextPrePage();
						changeSlider(getContent().pageIndex);
						/* init image had draw */
						initDisplayMarking();
						/* change title of page */
						if(dataPageTitle[getContent().pageIndex]){						
							if(dataPageTitle[getContent().pageIndex] != ''){ 						
								document.title = contentName + '_' + dataPageTitle[getContent().pageIndex];
							}else{							
								document.title = contentName;
							}		
						}else{
								document.title = contentName;
						}
						/* set end log */
						SetEndLog(contentID);			
					}
				 });		
	}
	
	
};	
/* Flip Previous Page Transition */
Transition.prototype.flipPreviousPage = function() {
	var w = $('#main').attr('width');
	removeObject();	
		
	if(getContent().hasPreviousPage()) {
		$('#main').animate(
				{left : w + 'px'}, 
				{ 
					duration : "slow",
					complete : function() {
						$('#main').css('left', - w + 'px');
						$('#main').animate(
							{left : marginX + 'px'}, 
							{duration : 'slow',
							complete: function(){
								isFirstLoad = true;
								screenFit(); 
								}
							});		
							
						//change pageIndex and image bookmark
						getContent().previousPage();
						initImageBookmark();
						checkExistNextPrePage();
						changeSlider(getContent().pageIndex);
						/* init image had draw */
						initDisplayMarking();
						/* change title of page */ 
						if(dataPageTitle[getContent().pageIndex]){
							if(dataPageTitle[getContent().pageIndex] != ''){ 						
								document.title = contentName + '_' + dataPageTitle[getContent().pageIndex];
							}else{							
								document.title = contentName;
							}		
						}else{
								document.title = contentName;
						}						
						
						/* set end log */
						SetEndLog(contentID);												
					}
				 });			
	}
};	

/* Flip  Page Transition */
Transition.prototype.flipToPage = function(index) {
	var w = $('#main').attr('width');		
	
	if(index == getContent().pageIndex){
		/* init image had draw */
		initDisplayMarking();		
	}else if(index < getContent().pageIndex){
		removeObject();		
		$('#main').animate(
				{left : w + 'px'}, 
				{ 
					duration : "slow",
					complete : function() {	
						$('#main').css('left', - w + 'px');
						$('#main').animate(
							{left : marginX + 'px'}, 
							{duration : 'slow',
							complete: function(){	
								isFirstLoad = true;							
								screenFit(); 
								}
							});
						//change pageIndex and image bookmark
						getContent().toPage(index);
						initImageBookmark();				
						checkExistNextPrePage();
						changeSlider(index);
						/* init image had draw */
						initDisplayMarking();	
						/* change title of page */ 
						if(dataPageTitle[index]){
							if(dataPageTitle[index] != ''){ 						
								document.title = contentName + '_' + dataPageTitle[index];
							}else{							
								document.title = contentName;
							}													
						}else{
								document.title = contentName;
						}
						/* set end log */
						SetEndLog(contentID);	
					}
				 }
			 );
		}else{			
			removeObject();
			$('#main').animate(
				{left : '-' + w + 'px'}, 
				{ 
					duration : "slow",
					complete : function() {	
						$('#main').css('left', w + 'px');
						$('#main').animate(
							{left : marginX + 'px'}, 
							{duration : 'slow',
							complete: function(){
								isFirstLoad = true;
								screenFit(); 
								}
							});	
						//change pageIndex and image bookmark
						getContent().toPage(index);
						initImageBookmark();				
						checkExistNextPrePage();
						changeSlider(index);
						/* init image had draw */
						initDisplayMarking();	
						/* change title of page */ 
						if(dataPageTitle[index]){
							if(dataPageTitle[index] != ''){ 						
								document.title = contentName + '_' + dataPageTitle[index];
							}else{							
								document.title = contentName;
							}													
						}else{
								document.title = contentName;
						}
						/* set end log */
						SetEndLog(contentID);												
					 }
			 });
		}
};	

/* remove all object when change page */
function removeObject(){
	$('#text').children().remove();
	
	/* close dialog */
	if(isOpenPopUpText == true){
		isOpenPopUpText = false;
		ClosePopupText();
	}
	
	$("#boxCopyText").hide();
	$("#boxSearching").hide();
	$("#boxIndex").hide();	
	$("#boxBookMark").hide();
	$('#pop_up_memo').hide();
	
	
	isDisplayBookMarkList = false;	
	isDisplayCopyText = false;
    isDisplayListIndex = false;	
		
	if($('#playvideo').children().length > 0){
		$('#playvideo').children().remove();
	}
	
	if($('#playaudio').children().length > 0){
		$('#playaudio').children().remove();
	}
	if($('#playaudiopage').children().length > 0){
		$('#playaudiopage').children().remove();
	}
		
	if(avwUserEnvObj.browser == 'firefox'){
		srcAudioType0 = '';
		TypeAudio0 = '';
		srcAudioType2 = '';
		TypeAudio2 = '';
	}else{	
		/* do nothing */
	}
	
	hideDialog();
	
	/* play background audio */
	playAllAudio();
	
	/* mediaType = 4 */
	mediaType4_changeImage = 0;
}


/* Initialize PageViewer Component */
function initializeViewerComponent(viewId) {
	
	/* add some tags we need.*/
	$('#'+viewId).prepend(		
		'<canvas id="main"></canvas>' +
		'<canvas id="offscreen"></canvas>' +
		'<canvas id="draw_canvas"></canvas>' +
		'<div id="playvideo"></div>' +
		'<div id="playaudio" style="width:0px; height:0px;"></div>' +
		'<div id="playaudiopage" style="width:0px; height:0px;"></div>' +
		'<div id="playaudioallpage" style="width:0px; height:0px;"></div>' +
		'<div id="text"></div>'
		);

	/* apply css style */
	$('#main').css(
		{
		
			'position' : 'absolute',
			'top' : marginY + 'px',
			'left' : marginX + 'px',			 	
			'box-shadow' : '3px 3px 14px #333',
			'background-color' : 'rgba(153, 153, 153, 0.7)',
			'border-radius' : '4px'
		}		
	);
	$('#offscreen').css(
		{
			'visibility' : 'hidden',
			'display' : 'none'
		}
	);
	$('#draw_canvas').css(
		{					
			'display' : 'none',
			'position' : 'absolute',
			'cursor': 'default'	
		}
	);
	
	$('#playvideo').css(
		{
			'position' : 'fixed',
			'top' : marginY + 'px',
			'left' : marginX + 'px'
		}
	);
	
	//set default event
	setDefaultEvent();
	
	/*init canvas value */
	canvas_draw = document.getElementById('draw_canvas');
	context_draw = canvas_draw.getContext('2d');
	canvas_main = document.getElementById('main');
    context_main = canvas_main.getContext('2d');
    canvas_offscreen = document.getElementById('offscreen');
    context_offscreen = canvas_offscreen.getContext('2d');
};

function onClick_CanvasMain(event){
	if(!cancelClick) {
		
			//change coordinates 
			var imagePt = screenToImage(event.pageX, event.pageY);
			var canvasWidth = $('#offscreen').width();		
		
			if((event.pageX - marginX) >= destRect.left && (event.pageX - marginX) <= destRect.right){
				if(isCopyMemo == true){

					CopyMemo(objEditMemo.id,contentID,changePageIndex(getPageIndex()),$('#divDialogMemo'),event.pageX,event.pageY,function(){
							//set flag change memo 
							ClientData.isChangedMemo(true);	
							
							/* refresh draw memo */						
							drawMemoOnScreen();										
						});
					
				}else{
					/* click add memo */
					if(ClientData.IsAddingMemo() == true){											
						AddMemo(contentID,changePageIndex(getPageIndex()),$('#divDialogMemo'),event.pageX,event.pageY,function(){
							//set flag change memo 
							ClientData.isChangedMemo(true);
							ClientData.IsAddingMemo(false);			
							
							/* refresh draw memo */						
							drawMemoOnScreen();
										
							
							 //change image	             							
		        			$('#imgaddmemo').removeClass();
					        $('#imgaddmemo').addClass('memoAdd');
						});	
						
					}else{		
						
						var isClickMemo = false;				
						
						if(isOpenPopUpText == true){
							isOpenPopUpText = false;
							ClosePopupText();
						}
						
						if(isOpenPopUpMemo == true){
							isOpenPopUpMemo = false;
							$("#pop_up_memo").hide();
						}
						
						/* click memo edit */
						if (ClientData.IsDisplayMemo() == true){	
							/* check exist object memo in mouse position */
							for(var nIndex = 0; nIndex < memoObjects.length; nIndex++){
								var hitPageObjMemo = memoObjects[nIndex];				
								if(hitPageObjMemo.hitTest(imagePt.x, imagePt.y)) {
									//console.log("PageObject Hit!! Id: " + hitPageObjMemo.id);
									//hitPageObjMemo.action();
									
									/* save object memo */
									objEditMemo = hitPageObjMemo;							
									var posMemoX = event.pageX;
									var posMemoY = event.pageY;
									
									if(imagePt.y > heightEachPage - $("#pop_up_memo").height()){
										posMemoY = posMemoY - $("#pop_up_memo").height();	
										
									}
									
									if(imagePt.x > widthEachPage - $("#pop_up_memo").width()){
										posMemoX = posMemoX - $("#pop_up_memo").width();	
									}
									
									/*display pop-up-memo */
									$("#pop_up_memo").css('top', posMemoY).css('left', posMemoX);
									$("#pop_up_memo").show();
									 
									/*set true for flag click memo */
									isClickMemo = true;
									isOpenPopUpMemo =true;
																	
								}
							}
						}	
						
						/* click pageObject */				
						if(isClickMemo == false){
							isClickLinkList = false;
							
							/* check exist object in mouse position */
							var hitPageObj = getContent().currentPage.hitTest(imagePt.x, imagePt.y);
							
							if(hitPageObj) {								
								//console.log("PageObject Hit!! Id: " + hitPageObj.id);
								hitPageObj.action();				
							}else{ 
							
								/* check click link list */
								for(var nIndex = 0; nIndex < webGetContentType4Objects.length; nIndex++){
									var hitPageObjLinkList = webGetContentType4Objects[nIndex];				
									if(hitPageObjLinkList.hitTest(imagePt.x, imagePt.y)) {
										//console.log("PageObject Hit!! Id: " + hitPageObjLinkList.id);
										hitPageObjLinkList.action();
										
										/*set true for flag click memo */
										isClickLinkList = true;
									}
								}
									
								if(isClickLinkList == false && ClientData.IsAddingMarking() == false){
									/* area next and prev page */
									var cwMain = $('#main').width();
									if(event.pageX > 0 && event.pageX < 300){
										prevPage_click();
									}else if(event.pageX > (cwMain - 300) && event.pageX < cwMain){								
										nextPage_click();
									}
								}
								
							}
						}															
					}
				}		
			}else{
				if(ClientData.IsAddingMarking() == false){
					var cwMain = $('#main').width();
					if(event.pageX > 0 && event.pageX < 300){
						prevPage_click();
					}else if(event.pageX > (cwMain - 300) && event.pageX < cwMain){								
						nextPage_click();
					}
				}
			}
		}
		cancelClick = false;	
}

function mouseMove_canvasMain(event){
	
	event.preventDefault();	
	/* base image move when userScale over 1 */
	if(moveFlag && userScale != 1) {			
		cancelClick = true;
		var mx;
		var my;
		// calc mouse moving distance
		if (avwUserEnvObj.os != "ipad" && avwUserEnvObj.os != "android") {
			mx = event.pageX - px;
		 	my = event.pageY - py;
		}else{
			mx = event.targetTouches[0].pageX - px; 
			my = event.targetTouches[0].pageY - py;
		}
		
		// scaling
		var sx = (destRect.right - destRect.left) / (srcRect.right - srcRect.left);
		var sy = (destRect.bottom - destRect.top) / (srcRect.bottom - srcRect.top);
		
		// calc scaling moving distance
		moveX = Math.round(-mx * sx);
		moveY = Math.round(-my * sy);
	
		
		// store current mouse point
		if (avwUserEnvObj.os != "ipad" && avwUserEnvObj.os != "android") {
			px = event.pageX;
			py = event.pageY;
		}else{
			px = event.targetTouches[0].pageX;
			py = event.targetTouches[0].pageY;	
		}
		
		// redraw
		flip();
		
		zoomVideo();
	}		
		
	var cwMain = $('#main').width();
	var chMain = $('#main').height();
	/* handle image prev on canvas */	
	if(getContent().hasPreviousPage()){
		$('#button_pre_canvas').mousemove(function(){
			if(getContent().hasPreviousPage()){
				
				$(this).css('opacity','1');
			}else{
				$(this).css('opacity','0');
			}
		});
		
		if(event.pageX > 0 && event.pageX < 300){				
			$('#button_pre_canvas').css('opacity','0.25');
		}else{
			$('#button_pre_canvas').css('opacity','0');
		}
	}else{
		$('#button_pre_canvas').css('opacity','0');			
	}
	
	/* handle image next on canvas */	
	if(getContent().hasNextPage()){
		$('#button_next_canvas').mousemove(function(){
			if(getContent().hasNextPage()){
				$(this).css('opacity','1');
			}else{
				$(this).css('opacity','0');
			}
		});
		
		if(event.pageX > (cwMain - 300) && event.pageX < cwMain){
			$('#button_next_canvas').css('opacity','0.25');
		}else{
			$('#button_next_canvas').css('opacity','0');
		}
	}else{
		$('#button_next_canvas').css('opacity','0');
	}

}

function mouseDown_CanvasMain(event){	
	moveFlag = true;
	/*if(event.originalEvent) {
		if(event.originalEvent.targetTouches) {
			if(event.originalEvent.targetTouches[0].pageX) {
			px = event.originalEvent.targetTouches[0].pageX;	
			}
		}			
	}*/
	
	// if(event.originalEvent.targetTouches[0].pageY){
		// py = event.originalEvent.targetTouches[0].pageY;
	// }
	
	//py = Number(event.originalEvent.targetTouches[0].pageY);
	if (avwUserEnvObj.os != "ipad" && avwUserEnvObj.os != "android") {
		px = event.pageX;
		py = event.pageY;
	}else{					
		px = event.targetTouches[0].pageX;
		py = event.targetTouches[0].pageY;		
	}
	
}

function mouseUp_CanvasMain(event){
	moveFlag = false;	
}

/* View Component setDefaultEvent */
function setDefaultEvent() {
	document.getElementById('main').addEventListener('click',onClick_CanvasMain,false);
	if(isTouchDevice() == true){		
		document.getElementById('main').addEventListener('touchstart',mouseDown_CanvasMain,false);
		document.getElementById('main').addEventListener('touchmove',mouseMove_canvasMain,false);
		document.getElementById('main').addEventListener('touchend',mouseUp_CanvasMain,false);
	}else{		
		document.getElementById('main').addEventListener('mousedown',mouseDown_CanvasMain,false);
		document.getElementById('main').addEventListener('mousemove',mouseMove_canvasMain,false);		
		document.getElementById('main').addEventListener('mouseup',mouseUp_CanvasMain,false);
		
		$('#main').mouseout(function(event) {
		if(moveFlag) {
			moveFlag = false;
		}
		
		$('#button_pre_canvas').css('opacity','0');	
		$('#button_next_canvas').css('opacity','0');
	});	
	}
	
	/*$('#main').bind("click",onClick_CanvasMain);	
	$('#main').bind("mousemove touchmove",mouseMove_canvasMain);	
	$('#main').bind("mousedown touchstart",mouseDown_CanvasMain);
	$('#main').bind("mouseup touchend",mouseUp_CanvasMain);
	
	
	document.getElementById('main').addEventListener('click',onClick_CanvasMain,false);
	document.getElementById('main').addEventListener('touchstart',mouseDown_CanvasMain,false);
	document.getElementById('main').addEventListener('mousedown',mouseDown_CanvasMain,false);
	document.getElementById('main').addEventListener('mousemove touchmove',mouseMove_canvasMain,false);
	document.getElementById('main').addEventListener('mouseup touchend',mouseUp_CanvasMain,false);*/
	
	
	
	
};

/*disable all control when click copy memo */
function disableControlsCopyMemo(){
	/* set cursor pointer*/
	$("#imgHome").css('cursor', 'default');
	$("#imgBack").css('cursor', 'default');
	$("#listbookmark").css('cursor', 'default');
	$("#imgbookmark").css('cursor', 'default');
	$("#listindex").css('cursor', 'default');
	$("#copytext").css('cursor', 'default');
	$("#imgmemo").css('cursor', 'default');
	$("#imgaddmemo").css('cursor', 'default');
	$("#imgmarking").css('cursor', 'default');
	$("#imgmarkingtoolbar").css('cursor', 'default');
	$("#firstpage").css('cursor', 'default');
	$("#prevpage").css('cursor', 'default');
	$("#nextpage").css('cursor', 'default');
	$("#lastpage").css('cursor', 'default');
	
	$('#imgHome').unbind('click');
	$('#imgHome').removeClass();
    $('#imgHome').addClass('home_off');
    
	/* back button */
	$('#imgBack').unbind('click');
	$('#imgBack').removeClass();
    $('#imgBack').addClass('back_off');
    
	/*list bookmark */
	$('#listbookmark').unbind('click');
    $('#listbookmark').removeClass();
    $('#listbookmark').addClass('bmList_off');
    
	$('#imgbookmark').unbind('click');
	$('#imgbookmark').removeClass();
    $('#imgbookmark').addClass('bmAdd_off');
	
	$('#listindex').unbind('click');
	$('#listindex').removeClass();
    $('#listindex').addClass('index_off');
    
	$('#copytext').unbind('click');
	$('#copytext').removeClass();
    $('#copytext').addClass('copy_off');
	
	/* imgage display memo */ 
	$('#imgmemo').unbind('click');
	$('#imgmemo').removeClass();
    $('#imgmemo').addClass('memoDisplay_off');
	
	/* image add memo */
	$('#imgaddmemo').unbind('click');
	$('#imgaddmemo').removeClass();
    $('#imgaddmemo').addClass('memoAdd_off');
	
	$('#imgmarking').unbind('click');
	$('#imgmarking').removeClass();
    $('#imgmarking').addClass('marking_off');
		
	/* check is loading page */
	$('#imgmarkingtoolbar').unbind('click');
	$('#imgmarkingtoolbar').removeClass();
	$('#imgmarkingtoolbar').addClass('markingToolbar_off');	
	
	$('#firstpage').unbind('click');
	$('#prevpage').unbind('click');
	$('#nextpage').unbind('click');
	$('#lastpage').unbind('click');
					
	$( "#slider_page" ).slider( "option", "disabled", true );							
	disable('#txtSearch','#txtSlider');
	
	$('#button_next_canvas').css('display','none');
	$('#button_pre_canvas').css('display','none');
}

/*enable all control when finish copy memo */
function enableControlsCopyMemo(){
	disableControlsCopyMemo();
	
	/* set cursor pointer*/
	$("#imgHome").css('cursor', 'pointer');
	$("#imgBack").css('cursor', 'pointer');
	$("#listbookmark").css('cursor', 'pointer');
	$("#imgbookmark").css('cursor', 'pointer');
	$("#listindex").css('cursor', 'pointer');
	$("#copytext").css('cursor', 'pointer');
	$("#imgmemo").css('cursor', 'pointer');
	$("#imgaddmemo").css('cursor', 'pointer');
	$("#imgmarking").css('cursor', 'pointer');
	$("#imgmarkingtoolbar").css('cursor', 'pointer');
	$("#firstpage").css('cursor', 'pointer');
	$("#prevpage").css('cursor', 'pointer');
	$("#nextpage").css('cursor', 'pointer');
	$("#lastpage").css('cursor', 'pointer');
	
	$('#imgHome').bind('click',imgHome_click);
	$('#imgHome').removeClass();
	$('#imgHome').addClass('home');
	
	$('#imgBack').bind('click',imgBack_click);
	$('#imgBack').removeClass();
	$('#imgBack').addClass('back');
		
	/*list bookmark */
	$('#listbookmark').bind('click',showListBookMark);
	$('#listbookmark').removeClass();
	if(isDisplayBookMarkList == true){
		$('#listbookmark').addClass('bmList_hover');
	}else{
		$('#listbookmark').addClass('bmList');
	}		
    
	$('#imgbookmark').bind('click',bookmarkPage);
	$('#imgbookmark').removeClass();
	var nBookmarkId = isExistBookmark(); 
	if(nBookmarkId == -1){
		$('#imgbookmark').addClass('bmAdd');
	}else{
		$('#imgbookmark').addClass('bmAdd_hover');
	}
	
	$('#listindex').bind('click',showListPageIndex);
	$('#listindex').removeClass();
	if(isDisplayListIndex == true){
		$('#listindex').addClass('index_hover');
	}else{
		$('#listindex').addClass('index');
	}	
	
	$('#copytext').bind('click',showCopyText);
	$('#copytext').removeClass();
	if(isDisplayCopyText == true){
		$('#copytext').addClass('copy_hover');
	}else{
		$('#copytext').addClass('copy');
	}	
	
	/* image memo */ 
	$('#imgmemo').bind('click',handleMemo);
	$('#imgmemo').removeClass();
	if(ClientData.IsDisplayMemo() == true){
		$('#imgmemo').addClass('memoDisplay_hover');
	}else{
		$('#imgmemo').addClass('memoDisplay');
	}	
	
	/* image add memo */
	$('#imgaddmemo').bind('click',handleAddMemo);
	$('#imgaddmemo').removeClass();
	if(ClientData.IsAddingMemo() == true){			
		$('#imgaddmemo').addClass('memoAdd_hover');
	}else{			
		$('#imgaddmemo').addClass('memoAdd');
	}	
	
	$('#imgmarking').bind('click',imgmarking_click);
	$('#imgmarking').removeClass();
	if(ClientData.IsDisplayMarking() == true){			
		$('#imgmarking').addClass('marking_hover');
	}else{			
		$('#imgmarking').addClass('marking');
	}	
	
	if(ClientData.serviceOpt_marking() == 'Y'){
		$('#imgmarkingtoolbar').bind('click',handleDrawCanvas);
		$('#imgmarkingtoolbar').removeClass();
    	if(ClientData.IsAddingMarking() == true){
  			$('#imgmarkingtoolbar').addClass('markingToolbar_hover');
      	}else{
  			$('#imgmarkingtoolbar').addClass('markingToolbar');
      	}      	
	}
		
	$('#firstpage').bind('click',firstPage_click);
	$('#prevpage').bind('click',prevPage_click);
	$('#nextpage').bind('click',nextPage_click);
	$('#lastpage').bind('click',lastPage_click);
	$( "#slider_page" ).slider( "option", "disabled", false );				
	enable('#txtSearch','#txtSlider');
	
	if(avwUserEnvObj.os != "ipad" && avwUserEnvObj.os != "android"){
		$('#button_next_canvas').css('display','block');
		$('#button_pre_canvas').css('display','block');
	}
	
	checkExistNextPrePage();
}

/* Change Coordinate Screen to Image */
function screenToImage(x, y) {
	
	var pt = new Point(0, 0);
	
	// get current position of mouse on screen	
	if(isFullScreen == true){
		pt.x = x - marginX;
		pt.y = y ;
	}else {
		pt.x = x - marginX;
		pt.y = y - marginY;
	}
	
	var sx = (destRect.right - destRect.left) / (srcRect.right - srcRect.left);
	var sy = (destRect.bottom - destRect.top) / (srcRect.bottom - srcRect.top);
	
	// change size image correct
	var nx = (pt.x - destRect.left) / sx;
	var ny = (pt.y - destRect.top) / sy;
	pt.x = Math.floor(nx + srcRect.left);
	pt.y = Math.floor(ny + srcRect.top);

	//console.log('screenToImage ' + srcRect.right + ' ' + srcRect.left + ' ' + destRect.right + ' ' + destRect.left + ' ' + x + ' ' + sx + ' ' + nx + ' ' + pt.x + ' ' + srcRect.bottom + ' ' + srcRect.top + ' ' + destRect.bottom + ' ' + destRect.top + ' ' + y + ' ' + sy + ' ' + ny + ' ' + pt.y);
	
	return pt;
};

/* grab viewer content object */
function getContent() {
	return content;
};

/* create page */
function createPage(pageNumber, pageImage, pageObjects, onCompleteFunc) {
	var page = new ContentPage(pageNumber, pageImage);
	/* add page Object */
	page.addPageObjects(pageObjects);
	
	/* load page */
	page.load(onCompleteFunc);

	return page;
};


/* Adjust page size */
function adjustPage(width, height) {
	/*$("#offscreen").attr('height', getContent().currentPage.image.height);
	$("#offscreen").attr('width', getContent().currentPage.image.width);	*/	
	 
	drawCanvas();	
	
};
/* draw Canvas  */
function drawCanvas() {
    var canvas = document.getElementById('offscreen');
    var context = canvas.getContext('2d');
    
    context.clearRect(0, 0, canvas.width, canvas.height);
    
	/* draw canvas */
	draw(context);
	
	/*screen to image */
	//flip(); 
};

/* draw memo */
function drawMemoOnScreen(){
	var canvas = document.getElementById('offscreen');
    var context = canvas.getContext('2d');
	
	if(ClientData.IsDisplayMemo() == true){					
		memoObjects = [];
		
		/*get data memo */
		var memoData = ClientData.MemoData();
		
		for(var nIndex = 0; nIndex < memoData.length; nIndex++){			
			if(memoData[nIndex].contentid == contentID
			&& memoData[nIndex].pageNo == changePageIndex(getPageIndex())){
				/* create object of memo */
				var memoObject = null;
				
				if(memoData[nIndex].posY > heightEachPage - 50){
					memoData[nIndex].posY = heightEachPage - 50;	
				}
				
				if(memoData[nIndex].posX > widthEachPage - 50){
					memoData[nIndex].posX = widthEachPage - 50;	
				}
				
				memoObject = new memo(
					nIndex,
					memoData[nIndex].posX,
					memoData[nIndex].posY,
					50,
					50,
					"img/memo.png"
				)
				
				memoObjects.push(memoObject);
				memoObject.drawMemo(context);					
			}				
		}							
	}
}

/* draw webGetContent4 */
function drawGetWebContent4(context){
	webGetContentType4Objects = [];	
	for (var nIndex = 0; nIndex < dataJsonType4.length; nIndex ++){		
		/* get data belong current page*/
		if(changePageIndex(getPageIndex()) == dataJsonType4[nIndex].page){
			var dataLinkList = dataJsonType4[nIndex].linkList;						
			for(var nLinkList = 0; nLinkList < dataLinkList.length; nLinkList++){
				/* create object webContentTyp4 */
				var dataObject = null;			
					
				dataObject = new webContentType4(
					nLinkList,
					dataLinkList[nLinkList].linkLocationX,
					dataLinkList[nLinkList].linkLocationY,
					dataLinkList[nLinkList].linkLocationWidth,
					dataLinkList[nLinkList].linkLocationHeight,
					"img/webGetContent4.png",
					dataLinkList[nLinkList].linkKind,
					dataLinkList[nLinkList].destURI,
					dataLinkList[nLinkList].destPageNumber					
				)				
				webGetContentType4Objects.push(dataObject);
				dataObject.drawObject(context);
				
			}
		}
	}
}

/* draw marking */
function drawMarkingOnScreen(){	
		
	var canvas = document.getElementById('offscreen');
    var context = canvas.getContext('2d');    
	var canvasWidth = $('#offscreen').width();		
	var canvasHeight = $('#offscreen').height();
	var indexMarking;
	
	if(ClientData.IsAddingMarking() == false && ClientData.IsDisplayMarking() == true){	
		var isExistDrawing = false;
			
		/* get data marking on local */
		var dataMarking = ClientData.MarkingData();		
		for(var nIndex = 0; nIndex < dataMarking.length; nIndex++){
			if(dataMarking[nIndex].contentid == contentID
			&& dataMarking[nIndex].pageNo == changePageIndex(getPageIndex())){		
				/*create image */											
				var img = new Image();				
				img.onload = function(){																																 					 					 				   											
					//context.drawImage(img,dataMarking[indexMarking].left,0,dataMarking[indexMarking].right - dataMarking[indexMarking].left,dataMarking[indexMarking].bottom - dataMarking[indexMarking].top,0,0,canvasWidth,canvasHeight);
					context.drawImage(img,0,0,canvasWidth,canvasHeight);					
					flip();
					 /* set flag */
					isExistDrawing = true;			 
					 /* draw memo */
					drawMemoOnScreen();										 				 	 						
				}
				img.src = dataMarking[nIndex].content;						 						
				indexMarking = nIndex;
			}
		}
		
		if(isExistDrawing == false){
			/* draw memo */
			drawMemoOnScreen();	
		}			
	}else{
		/* draw memo */	
		drawMemoOnScreen();	
	}	
}

/*draw canvas*/
function draw(context) {
    context.save();    
     
    /* draw page objects */
    getContent().currentPage.drawPage(context);               
    
    
   
   /* draw memo */
   //drawMemoOnScreen();
   
   context.restore();	   

};


var leftCanvas = 0;
var topCanvas = 0;
var count = 0
/* display to screen */
function flip() {
    var canvas = document.getElementById('main');
    var offscreen = document.getElementById('offscreen');
    var context = canvas.getContext('2d');    
    var contextOffscreen = offscreen.getContext('2d');    
	// init rect
	if(srcRect.width == 0) {
		srcRect = new Rect(0, 0, offscreen.width, offscreen.height);		
	}
	
	// display rect
	srcRect.add(moveX, moveY);
	
	// fix rect
	if(srcRect.left < 0) {
		srcRect.left = 0;
		srcRect.right = srcRect.left + srcRect.width;
	} else if(srcRect.right > offscreen.width) {
		srcRect.left = offscreen.width - srcRect.width;
		srcRect.right = offscreen.width
	}
	if(srcRect.top < 0) {
		srcRect.top = 0;
		srcRect.bottom = srcRect.top + srcRect.height;
	} else if(srcRect.bottom > offscreen.height) {
		srcRect.top = offscreen.height - srcRect.height;
		srcRect.bottom = offscreen.height;
	}
	
	// target rect
    var width = offscreen.width;
    var height = offscreen.height;
	var aspect = offscreen.width / offscreen.height;
	if(canvas.width > canvas.height) {
		height = canvas.height;
		width = height * aspect;
		if(width > canvas.width) {
			width = canvas.width;
			height = width / aspect;
		}
	} else {
		width = canvas.width;
		height = width / aspect;
	}
	
	if(height > canvas.height){
		var size = canvas.height / height;
		height = canvas.height;
		width = width * size;
	}
	
	var destX = 0, destY = 0;
	destX = (canvas.width / 2) - (width / 2);
	destY = (canvas.height / 2) - (height / 2);
	
	if(destX < 0){
		destX = 0;	
	}
	if(destY < 0){
		destY = 0;	
	}
	
	if(srcRect.left < 0){
		srcRect.left = 0;	
	}
	if(srcRect.top < 0){
		srcRect.top = 0;	
	}

	destRect = new Rect(destX, destY, width, height);
	
	/* get position for drawing canvas*/
	if(isFirstLoad == true){
		nPositionCanvas.left = destRect.left;
		nPositionCanvas.right = destRect.right;
		nPositionCanvas.top = destRect.top;
		nPositionCanvas.bottom = destRect.bottom;	
		
		$("#draw_canvas").attr('height', destRect.bottom - destRect.top)
			  .attr('width', destRect.right - destRect.left)
			   .css('top', destRect.top + marginY - 1)
			   .css('left',destRect.left + marginX - 1);
 
		isFirstLoad = false;
	}
	
	leftCanvas = destX;
	topCanvas = destY;
	// change scale
	scaleX = offscreen.width / width;
	scaleY = offscreen.height / height;
	
	// draw canvas		
	context.clearRect(0, 0, canvas.width, canvas.height);
	context.save();

	context.drawImage(offscreen,
					 	  srcRect.left, srcRect.top, srcRect.right - srcRect.left, srcRect.bottom - srcRect.top, 
					  destRect.left, destRect.top, width, height);
	context.restore();
};

/* mediaType = 1 and actionType = 0 linkURL object*/
var imageNoAction = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible) {	
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null, null);  
	this.action = function(){
		//no action
	}                	              	                                                    
};
imageNoAction.prototype = new PageObject();   

/* mediaType=1  Video object : extends PageObject */
var videoType1 = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, mediaFile,media) {	
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null, null);
	this.action = function() {
		if(media != ''){
			/* stop all audio on page */		
			stopAllAudio();		
			
			createVideoFulScreen(mediaFile,id)
			
			$("#btnCloseVideo").click(function (e){
				hideDialog();
				
				/* play audio  */
				playAllAudio();			
			}); 
			
			showDialog(true);  
		}  
	}
};
videoType1.prototype = new PageObject();

/* mediaType=1 and actionType = 2  Audio object : extends PageObject */
var audioType1 = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, audioFile) {
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null, null);
	this.action = function() {		
		createAudio(audioFile,"0")
	}
};
audioType1.prototype = new PageObject();

/* mediaType = 1 and actionType = 3 linkURL object*/
var linkURL = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, actionFunction,linkUrl,browserType) {	
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, actionFunction, null);                  
	this.action = function() {  		                 
	 	window.open(linkUrl,"_blank","new window");	  
	 	}     	                                                    
};
linkURL.prototype = new PageObject();                                                                                                         

/* mediaType = 1 and actionType = 4 imagePreview object*/
var imagePreview = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, imagePreview) {    
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null, null);                  
	this.action = function() {  
		
		createImagePreview(); 
		
		showImagePreview($('#divImagePreview'), imagePreview);	
		                                                                        	
		$("#btnClose").live('click',hideDialog);
		
	  
		showDialog(true);                                         
	}                                                                         
};
imagePreview.prototype = new PageObject();                                                                                                         

function showDialog(modal)
{
	$("#overlay").show();
	$("#dialog").fadeIn(300);

	if (modal)
	{
		$("#overlay").unbind("click");
	}
	else
	{
		$("#overlay").click(function (e)
		{
			hideDialog();
		});
	}
}
		
function hideDialog()
{
	$("#overlay").hide();
	$("#dialog").fadeOut(300);
	$('#dialog').children().remove();
	initFlg = true;
}

function createImagePreview(){
	$('#dialog').prepend(
		'<h1 style="height:25px;background:url(./img/viewer/pophdbg.png) 0 0 repeat" >' +
		'<img src="img/viewer/x.png" style="margin:3px 3px 0px 21px" id="btnClose" class="align_right" ></img>' +
		'</h1>' +
		'<div id="divImagePreview" class="content_image_preview">' +				
		'</div>'
	);	
	$('#dialog').css('box-shadow','0px 0px 3px 1px #666');
	$('#dialog').css('border','solid 1px #666');
	
	$('#dialog').draggable({ handle: "h1" });
}

/* mediaType = 1 and actionType = 5 jumpPage object*/
var jumpPage = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, jumpPage) {    
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null,null);                  
	this.action = function() {  
		if((Number(jumpPage) - 1) != getPageIndex()){				
			changePage(Number(jumpPage) - 1);		     
		}                               
	}                                                                         
};
jumpPage.prototype = new PageObject(); 

/* mediaType = 1 and actionType = 6 send mail object*/
var sendMail = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, emailSubject, emailAddress) {    
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null,null);                  
	this.action = function() {  		
		MailTo(emailAddress,emailSubject);                                   
	}                                                                         
};
sendMail.prototype = new PageObject(); 

/* mediaType = 1 and actionType = 8 open popup object*/
var openPopUp = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, content) {    
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null,null);                  
	this.action = function() {  
		isOpenPopUpText = true;
		var pt1 = imageToScreen(x, y);	
		OpenPopupText(pt1.x, pt1.y + h/2, content, $('#dialogPopUp'), $('#arrow'));						                                    
	}                                                                         
};
openPopUp.prototype = new PageObject(); 


/* mediaType = 1 and actionType = 10 move to Content object*/
var moveToContent = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, contentId, pageNo) {    
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null,null);                  
	this.action = function() {  				
		/* set end log */
		SetEndLog(contentID);
		RegisterLog();
			
		ClientData.common_preContentId(contentId);
		ClientData.common_prePageNo(pageNo);
		
		/* store old page */
		var oldContent = {oldContentID: contentID, oldPageIndex: getPageIndex()};		
				
		var dataJump = ClientData.JumpQueue();
		
		dataJump.push(oldContent);
		
		ClientData.JumpQueue(dataJump);

		avwScreenMove(ScreenIds.ContentView)               
	}                                                                         
};
moveToContent.prototype = new PageObject(); 

/* mediaType=2  Video object : extends PageObject */
var videoType2 = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, mediaFile,media,embed) {

	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null,embed);
	this.action = function() {						
		/*stop audio on page */
		stopAllAudio();		
		if(embed == "1"){
			showVideoObject(x, y, w, h,mediaFile,false);		
		}else{
			if(media != ''){				
				$('#playvideo').children().remove();
				
				createVideoFulScreen(mediaFile,id)
				
				$("#btnCloseVideo").click(function (e){
					hideDialog();
					/*play audio if exist */
					playAllAudio();
				}); 
				
				showDialog(true);
			}
		}    
	}
};
videoType2.prototype = new PageObject();

/* create video on page */
function createVideoFulScreen(video,id){
	$('#dialog').css('box-shadow','0px 0px 3px 1px #666');
	$('#dialog').css('border','solid 1px #666');
	$('#dialog').draggable({ handle: "h1" });
	
	$('#dialog').children().remove();	
	if(ClientData.userOpt_videoMode() == "1"){/* loop video */
		$('#dialog').html(		
			'<h1 style="height:25px;background:url(./img/viewer/pophdbg.png) 0 0 repeat">' +
			'	<img src="img/viewer/x.png" style="margin:3px 3px 0px 21px" id="btnCloseVideo" class="align_right" ></img>' +
			'</h1>' +
			'		<div id="videoMedia' +id+ '">' +
			'			<video style="height:90%; width:100%;" class="mov" id="videoOnPopUp" autoplay controls loop>'	+
			'			 <source src="' + video + '" type="video/mp4"> ' +	
	  		'    </video>' +	
			'		</div>' 
			);
	}else{
		$('#dialog').html(		
			'<h1 style="height:25px;background:url(./img/viewer/pophdbg.png) 0 0 repeat">' +
			'	<img src="img/viewer/x.png" style="margin:3px 3px 0px 21px" id="btnCloseVideo" class="align_right" width=25px" ></img>' +
			'</h1>' +
			'		<div id="videoMedia' +id+ '">' +
			'			<video style="height:90%; width:100%;" id="videoOnPopUp" class="mov" autoplay controls>'	+
			'			 <source src="' + video + '" type="video/mp4"> ' +	
	  		'    </video>' +	
			'		</div>' 
			);
	}
    $('#videoOnPopUp').css('height', ($('#dialog').height() - 25) + "px" );
	//showControlsVideo($('#videoOnPopUp'));
}

/* handle display controls of video */
function showControlsVideo(target) {
          $(target).click(function () {
              $(target).attr('controls', '');
          });
          $(target).mouseout(function () {
              $(target).removeAttr('controls');
          });
          $(target).removeAttr('controls');
      }


/* show video*/
function showVideoObject(x, y, width, height, src, isFullscreen) {

	var pt1 = imageToScreen(x, y);
	var pt2 = imageToScreen(x + width, y + height);
	
	if(isFullScreen == true){
		pt2.y = pt2.y - marginY;
		pt1.y = pt1.y - marginY;	
	}
	
	//getPosVideo(x, y, (pt2.x - pt1.x), (pt2.y - pt1.y));
	getPosVideo(x, y, width, height);
	
	if(isFullscreen === false){	
		$('#playvideo').attr('z-order', '1000');
		$('#playvideo').css('left', pt1.x + 'px');
		$('#playvideo').css('top', pt1.y + 'px');
		$('#playvideo').children().remove();
		
		if(ClientData.userOpt_videoMode() == "1"){/* loop video */
			$('#playvideo').html('<video class="mov" id="videoOnPage" ' +
							 'width="' + (pt2.x - pt1.x) + '" ' +
							 'height="' + (pt2.y - pt1.y) + '" ' +
							 'autoplay controls loop>' +
							 '	 <source src="' + src + '&key=' + (new Date()).toIdString() + '" type="video/mp4"> ' +	
	  						 '    <source src="' + src + '&key=' + (new Date()).toIdString() + '" type="video/ogg"> ' +		
							 '</video>' 
				);
		}else{
			$('#playvideo').html('<video class="mov" id="videoOnPage"' +
							 'width="' + (pt2.x - pt1.x) + '" ' +
							 'height="' + (pt2.y - pt1.y) + '" ' +
							 'autoplay controls>' +
							 '	 <source src="' + src + '&key=' + (new Date()).toIdString() + '" type="video/mp4"> ' +	
	  						 '    <source src="' + src + '&key=' + (new Date()).toIdString() + '" type="video/ogg"> ' +		
							 '</video>' 
				);
		}
	}else {
		$('#playvideo').attr('z-order', '1000');		
		$('#playvideo').html('<video class="mov" id="videoOnPage" src="' + src + '" ' +						 
						 'autoplay controls></video>');
	}
	
	//showControlsVideo($('#videoOnPage'));

};

/* mediaType=3  Audio object : extends PageObject */
var audioType3 = function(mediaType, actionType, id, imageUrl, x, y, w, h, visible, audioFile,playType) {
	
	if(playType == "0"){
		this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, null,null);	
		this.action = function() {		
			createAudio(audioFile, playType);
		}
	}else{
		createAudio(audioFile, playType);		
		this.setup(mediaType, actionType, id, null, x, y, w, h, visible, null,null);	
	}
};
audioType3.prototype = new PageObject();


/* stop all audio on page */
function stopAllAudio(){
	if(avwUserEnvObj.browser == 'firefox'){	
		$('#playaudiopage').children().remove();
		$('#playaudioallpage').children().remove();
		$('#playaudio').children().remove();
	}else{
		if(document.getElementById("play_audio_0") != undefined){
			document.getElementById("play_audio_0").pause();
		}
		
		if(document.getElementById("play_audio_1") != undefined){
			document.getElementById("play_audio_1").pause();
		}
		
		if(document.getElementById("play_audio_2") != undefined){
			document.getElementById("play_audio_2").pause();
		}
	}
}

/* play all audio on page */
function playAllAudio(){
	if(avwUserEnvObj.browser == 'firefox'){
		if(srcAudioType0 != ''){
			createAudio(srcAudioType0,TypeAudio0);
		}else if(srcAudioType2 != ''){
			createAudio(srcAudioType2,TypeAudio2);
		}else if(TypeAudio1 != ''){
			createAudio(srcAudioType1,TypeAudio1);
		}
	}else{
		if(document.getElementById("play_audio_0") != undefined){
			document.getElementById("play_audio_0").play();
		}else if(document.getElementById("play_audio_2") != undefined){		
			document.getElementById("play_audio_2").play();
		}else if(document.getElementById("play_audio_1") != undefined){		
			document.getElementById("play_audio_1").play();
		}
	}
}

var srcAudioType0 = '';
var TypeAudio0;
var srcAudioType1 = '';
var TypeAudio1;
var srcAudioType2 = '';
var TypeAudio2;

/* create audio on page */
function createAudio(audio,type){	
	if(audio != ''){
		if(type == "0"){ /* play when click */		
			stopAllAudio();		
			
			TypeAudio0 = type;
			srcAudioType0 = audio;
			
			if(ClientData.userOpt_musicMode() == "1"){
				if(avwUserEnvObj.browser == 'firefox'){
					$('#playaudio').html(
						'	 <embed id="play_audio_0" src="' + audio + '&key=' + (new Date()).toIdString() + '" autostart="true" loop="true" hidden="true"> </embed>' 						
					);
				}else{			
					$('#playaudio').html(
						'	<audio  id="play_audio_0" loop="loop"  autoplay ="autoplay">' +
						'		 <source src="' + audio + '&key=' + (new Date()).toIdString() + '" type="audio/mpeg" />' +
						'	</audio>'
					);
				}
			}else{
				if(avwUserEnvObj.browser == 'firefox'){
					$('#playaudio').html(
						'	 <embed id="play_audio_0" src="' + audio + '&key=' + (new Date()).toIdString() + '" autostart="true" hidden="true"> </embed>' 						
					);
				}else{
					$('#playaudio').html(
						'	<audio  id="play_audio_0" autoplay ="autoplay">' +
						'		 <source src="' + audio + '&key=' + (new Date()).toIdString() + '" type="audio/mpeg" />' +
						'	</audio>'
					);
				}
			}
			
		}else if(type == "1"){ /* play all page */	
			
			TypeAudio1 = type;
			srcAudioType1 = audio;
			
			 if(document.getElementById("play_audio_1") != undefined){
				 //document.getElementById("play_audio_1").load();
			 }else{	
			 	
			  	if(ClientData.userOpt_musicMode() == "1"){	
			  		if(avwUserEnvObj.browser == 'firefox'){
						$('#playaudioallpage').html(
							'	 <embed id="play_audio_1" src="' + audio + '&key=' + (new Date()).toIdString() + '" autostart="true" loop="true" hidden="true"> </embed>' 						
						);
					}else{			
						 $('#playaudioallpage').html(
							 '	<audio  id="play_audio_1" loop="loop" autoplay ="autoplay">' +
							 '		 <source src="' + audio + '&key=' + (new Date()).toIdString() + '" type="audio/mpeg" />' +
							 '	</audio>'	
						 );		
					}			
				}else{
					if(avwUserEnvObj.browser == 'firefox'){
						$('#playaudioallpage').html(
							'	 <embed id="play_audio_1" src="' + audio + '&key=' + (new Date()).toIdString() + '" autostart="true" hidden="true"> </embed>' 						
						);
					}else{
						$('#playaudioallpage').html(
							 '	<audio  id="play_audio_1" autoplay ="autoplay">' +
							 '		 <source src="' + audio + '&key=' + (new Date()).toIdString() + '" type="audio/mpeg" />' +
							 '	</audio>'	
						 );		
					}
				}
			 }
		}
		else if(type == "2"){ /* play on page */	
			stopAllAudio();			
			
			TypeAudio2 = type;
			srcAudioType2 = audio;
			
			if(ClientData.userOpt_musicMode() == "1"){	
				if(avwUserEnvObj.browser == 'firefox'){
						$('#playaudiopage').html(
							'	 <embed id="play_audio_2" src="' + audio + '&key=' + (new Date()).toIdString() + '" autostart="true" loop="true" hidden="true"> </embed>' 						
						);
				}else{						
					$('#playaudiopage').html(
						'	<audio  id="play_audio_2" loop="loop" autoplay ="autoplay">' +
						'		 <source src="' + audio + '&key=' + (new Date()).toIdString() + '" type="audio/mpeg" />' +
						'	</audio>'	
					);		
				}
			}else{
				if(avwUserEnvObj.browser == 'firefox'){
						$('#playaudiopage').html(
							'	 <embed id="play_audio_2" src="' + audio + '&key=' + (new Date()).toIdString() + '" autostart="true" hidden="true"> </embed>' 						
						);
				}else{
					$('#playaudiopage').html(
						'	<audio  id="play_audio_2" autoplay ="autoplay">' +
						'		 <source src="' + audio + '&key=' + (new Date()).toIdString() + '" type="audio/mpeg" />' +
						'	</audio>'	
					);		
				}
			}					
		}
	}else{
		stopAllAudio();	
	}
	
}

/* mediaType=4  list image object : extends PageObject */
var listImage = function(mediaType, id, imageUrl, x, y, w, h, visible, imageObjects,triggerType) {
	this.setup(mediaType, null, id, imageUrl, x, y, w, h, visible, null, null);	
	this.action = function() {			
		if(triggerType == '1'){
			mediaType4_changeImage ++;
			
			/* check index bigger than length object */
			if(mediaType4_changeImage > (imageObjects.length - 1)){
				mediaType4_changeImage = 0;		
			}
			
			/* draw image */
			var imageObj = new Image();			    		
			imageObj.onload = function() {					
				var canvasObject = document.getElementById("offscreen");
				var contextObject = canvasObject.getContext("2d");										
			    contextObject.drawImage(imageObj, x, y, w, h);	    				    
			    flip();	  	 	 
			 };
			 imageObj.src = imageObjects[mediaType4_changeImage].fileName;	
		}		
	}
};
listImage.prototype = new PageObject();

/* mediaType=5 Replace Video object : extends PageObject */
var listVideo = function(mediaType, id, imageUrl, x, y, w, h, visible, videoObjects) {	
	this.setup(mediaType, null, id, imageUrl, x, y, w, h, visible, null, null);	
	this.action = function() {		
	}
};
listVideo.prototype = new PageObject();

/* mediaType=6 Trigger object : extends PageObject */
var trigger = function(mediaType, actionType, id, imageUrl, x, y, w, h,index,target, visible, actionFunction) {
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, actionFunction, null);
	this.action = function() {				
		// draw object		
		/*if(isObjType4 == true){			
			for(var nIndex = 0; nIndex < objType4.imageObjects.length; nIndex++) {
				if(objType4.imageObjects[nIndex].index == index){	
					alert(target);		
					changeImageType4(objType4,nIndex);	
					break;				
				}
			}
		}else if(isObjType5 == true){			
			for(var nIndex = 0; nIndex < objType5.videoObjects.length; nIndex++) {				
				if(objType5.videoObjects[nIndex].index == index){	
													
					showVideoObject(objType5.x, objType5.y, objType5.width, objType5.height,objType5.videoObjects[nIndex].fileName,false);
					
					break;					
				}
			}
		}*/		
		for(var nIndex = 0; nIndex < objType4_5.length; nIndex++) {
			for(var nIndex1 = 0; nIndex1 < objType4_5[nIndex].dataObjects.length; nIndex1++) {				
				if(objType4_5[nIndex].dataObjects[nIndex1].index == index
					&& objType4_5[nIndex].id == target){
						if(objType4_5[nIndex].mediaType == '4'){	/*image*/											
							changeImageType4(objType4_5,nIndex,nIndex1);
						}else{   /* video */
							stopAllAudio();	
							showVideoObject(objType4_5[nIndex].x, objType4_5[nIndex].y, objType4_5[nIndex].width, objType4_5[nIndex].height, objType4_5[nIndex].dataObjects[nIndex1].fileName,false);
						}	
						break;				
				}
			}
		}
	}
};
trigger.prototype = new PageObject();

/* change main image of media type 4 */
function changeImageType4(objTarget,nIndex,nIndex1){		
	var imageObj = new Image();			    		
	imageObj.onload = function() {	
		var canvasObject = document.getElementById("offscreen");
		var contextObject = canvasObject.getContext("2d");										
	    contextObject.drawImage(imageObj, objTarget[nIndex].x, objTarget[nIndex].y, objTarget[nIndex].width, objTarget[nIndex].height);	    				    
	    flip();	  	 	 
	 };
	 imageObj.src = objTarget[nIndex].dataObjects[nIndex1].fileName;	
}

/* mediaType = 1 write text object*/
var text = function(mediaType, actionType, id, x, y, w, h, visible, actionFunction, content) {    
	//var imageUrl = displayText(x,y,w,h,id,content);
	var imageUrl = getTextObjectImage(w, h, content)
	this.setup(mediaType, actionType, id, imageUrl, x, y, w, h, visible, actionFunction, null);                                                                 
};
text.prototype = new PageObject();

/*
var pxText;
var pyText;
*/
/* display text on canvas*/
/*
function displayText(x, y, width, height,id,content) {
	
	var canvasObject = document.getElementById("offscreen");
	var contextObject = canvasObject.getContext("2d");
	var dataHtml = "<div align='center'><font face='ＭＳ Ｐゴシック' style='LETTER-SPACING:-0.2px;LINE-HEIGHT:120%;FONT-SIZE:11px' color='#333333' letterspacing='0' kerning='1'><font style='LETTER-SPACING:-0.2px;LINE-HEIGHT:120%;FONT-SIZE:16px' color='#cc0000'><u>TEXT</u></font></font></div><div align='center'><font face='ＭＳ Ｐゴシック' style='LETTER-SPACING:-0.2px;LINE-HEIGHT:120%;FONT-SIZE:11px' color='#333333' letterspacing='0' kerning='1'><font style='LETTER-SPACING:-0.2px;LINE-HEIGHT:120%;FONT-SIZE:16px' color='#cc0000'><u>TEXT</u></font></font></div>";
	var data = "<svg xmlns='http://www.w3.org/2000/svg' width='196' height='261'>" +
             "<foreignObject width='100%' height='100%'>" +
               "<div xmlns='http://www.w3.org/1999/xhtml' style='font-size:40px'>" +
                dataHtml +
               "</div>" +
             "</foreignObject>" +
           "</svg>";
	var DOMURL = self.URL || self.webkitURL || self;
	//var img = new Image();
	var svg = new Blob([data], {type: "image/svg+xml;charset=utf-8"});
	var url = DOMURL.createObjectURL(svg);	  

	return url;
};
*/

function imageToScreen(x, y) {
	
	var pt = new Point(0, 0);
	
	// scale vertical and horizontal 
	var sx = (destRect.right - destRect.left) / (srcRect.right - srcRect.left);
	var sy = (destRect.bottom - destRect.top) / (srcRect.bottom - srcRect.top);
	 
	pt.x = Math.round(sx * (x - srcRect.left)) + destRect.left + marginX;
	pt.y = Math.round(sy * (y - srcRect.top)) + destRect.top + marginY;
	
	return pt;
};

/* Adjust Component size */
function sizingNotFull(width, height) {
	//adjust size of canvas using for draw		
	if(isTouchDevice() == true){
		$("#main") .css('top', marginY);
		$("#main").attr('height', height - (marginY * 2))
		  .attr('width', width - (marginX * 2));
		  
	}else{
			$("#main").attr('height', height - (marginY * 2))
			  .attr('width', width - (marginX * 2))
			   .css('top', marginY);
	}
	
	$("#draw_canvas").css('cursor', 'default');	
};

/* Adjust Component size */
function sizingFullSize(width, height) {
	// adjust size of body
	if(isTouchDevice() == true){
		$("#main") .css('top', '0px');
		$("#main").attr('height', height)
				   .attr('width', width - (marginX * 2));				   
	}else{
		$("#main").attr('height', height)
				   .attr('width', width - (marginX * 2))
				   .css('top', '0px');
	}
	
	//adjust size of canvas using for draw
	
	$("#draw_canvas").css('cursor', 'default');

};

/* zoom video */
var pxVideo;
var pyVideo;
var wVideo;
var hVideo;
var videoScale;

function getPosVideo(px,py,width,height,scale){
	pxVideo = px;
	pyVideo = py
	wVideo = width;
	hVideo = height;
	videoScale = scale;
}

function zoomVideo(){
	 var objVideo = document.getElementsByTagName('video');	 
	 if(objVideo.length >0){
	 	 var pt1 = imageToScreen(pxVideo, pyVideo);
	 	 var pt2 = imageToScreen(pxVideo + wVideo, pyVideo + hVideo);
	 
		 if(isFullScreen == true){
			pt2.y = pt2.y - marginY;
			pt1.y = pt1.y - marginY;				
		}
	 	 $('#playvideo').attr('z-order', '1000');	

	 	 $('#playvideo').css('left', pt1.x + 'px');
		 $('#playvideo').css('top', pt1.y + 'px');
	
		 $(objVideo[0]).attr('width',pt2.x - pt1.x);
		 $(objVideo[0]).attr('height',pt2.y - pt1.y);
		 /*$('#playvideo').html('<video class="mov" src="' + src + '" ' +
						 'width="' + (pt2.x - pt1.x) + '" ' +
						 'height="' + (pt2.y - pt1.y) + '" ' +
						 'autoplay controls></video>');
		*/
	 }	 
}

/* zoomIn event */
function zoomIn() {
	userScale += scaleDelta;
	
	if(userScale > 4){
		userScale = 4;
	}
	
	changeScale(userScale);
	
	flip();
	
	/* zoom video */
	zoomVideo();			 
};

/* zoomOut event */
function zoomOut() {
	userScale -= scaleDelta;
	if(userScale < 1) {
		userScale = 1;
	}
	changeScale(userScale);
	
	flip();
	
	/* zoom video */
	zoomVideo()
};

/* restore screen size */
function screenFit() {
	
    var canvas = document.getElementById('main');
    var offScreen = document.getElementById('offscreen');

	srcRect = new Rect(0, 0, offScreen.width, offScreen.height);
	userScale = 1;
	moveX = 0;
	moveY = 0;
	
	flip();
	
	/* zoom video */
	zoomVideo();
};


/* change Scale screen*/
function changeScale(scale) {
    var canvas = document.getElementById('main');
    var offScreen = document.getElementById('offscreen');
	
	var sc = srcRect.center();
	
	var sw = offScreen.width, sh = offScreen.height;
	sw /= scale;
	sh /= scale;
	
//	srcRect.left = sc.x - Math.floor(sw / 2);
//	srcRect.top = sc.y - Math.floor(sh / 2);
//	srcRect.right = srcRect.left + sw;
//	srcRect.bottom = srcRect.top + sh;
//	srcRect.width = sw;
//	srcRect.height = sh;
	srcRect.left = sc.x - Math.round(sw / 2);
	srcRect.top = sc.y - Math.round(sh / 2);
	srcRect.right = Math.round(srcRect.left + sw);
	srcRect.bottom = Math.round(srcRect.top + sh);
	srcRect.width = srcRect.right - srcRect.left;
	srcRect.height = srcRect.bottom - srcRect.top;
};

/* change Scale screen*/
function changeScaleDrawCanvas(scale) {
    var canvas = document.getElementById('draw_canvas');
    var offScreen = document.getElementById('draw_canvas');
	
	var sc = srcRect.center();
	
	var sw = offScreen.width, sh = offScreen.height;
	sw /= scale;
	sh /= scale;	
	srcRect.left = sc.x - Math.floor(sw / 2);
	srcRect.top = sc.y - Math.floor(sh / 2);	
	srcRect.right = srcRect.left + sw;
	srcRect.bottom = srcRect.top + sh;
	srcRect.width = sw;
	srcRect.height = sh;
};

function encode(data) {
  var str = String.fromCharCode.apply(null,data);
  return btoa(str).replace(/.{76}(?=.)/g,'$&\n');
 }


/* ----------------------------------- Create memo --------------------------------*/

var memoObjects =[];

/* constructor */
var MemoObject = function(id,x,y,width,heigth,imageUrl,action) {
	
};

/* setup each properties */
MemoObject.prototype.setup = function(id,x,y,w,h,imageUrl) {
	this.id = id;
	this.x = x;
	this.y = y;
	this.width = w;
	this.height = h;
	this.imageUrl = imageUrl;	
};

/* prototype hitTest */
MemoObject.prototype.hitTest = function(px, py) {
	if(this.x <= px && px <= (this.x + this.width)) {
		if(this.y <= py && py <= (this.y + this.height)) {
			return true;
		}
	}
	return false;
};

/* draw Memo */
MemoObject.prototype.drawMemo = function(context) {	
	var posX = this.x;
	var posY = this.y;
	var objWidth = this.width;
	var objHeigth = this.height;	
	var imageObj = new Image();		
		    		
	imageObj.onload = function(){			
		context.drawImage(imageObj, posX, posY, objWidth, objHeigth);
		flip();						
	}
	imageObj.src = this.imageUrl;
	
};

/* create memo object */
var memo = function(id,x, y, w, h,imageUrl) {			
	this.setup(id,x, y, w, h,imageUrl);	
	this.action = function() {					
		EditMemo(id, 0, 0, $('#divDialogMemo'), function() {									
			/*set flag change memo */
			ClientData.isChangedMemo(true);						
		});		
		
	}
};
memo.prototype = new MemoObject();

/*click showEditMemo() */
function showEditMemo(){
	objEditMemo.action();
}

/*click copyMemo() */
function copyMemo(){
	isCopyMemo = true;
	disableControlsCopyMemo();
	$("#pop_up_memo").hide();
}


/* ----------------------------------- End create memo -----------------------------*/

/* ----------------------------------- Create webGetContentType4 --------------------------------*/

var webGetContentType4Objects =[];

/* constructor */
var WebGetContentTyp4Object = function(id,x,y,width,heigth,imageUrl,action) {
	
};

/* setup each properties */
WebGetContentTyp4Object.prototype.setup = function(id,x,y,w,h,imageUrl) {
	this.id = id;
	this.x = x;
	this.y = y;
	this.width = w;
	this.height = h;
	this.imageUrl = imageUrl;	
};

/* prototype hitTest */
WebGetContentTyp4Object.prototype.hitTest = function(px, py) {
	if(this.x <= px && px <= (this.x + this.width)) {
		if(this.y <= py && py <= (this.y + this.height)) {
			return true;
		}
	}
	return false;
};

/* draw Memo */
WebGetContentTyp4Object.prototype.drawObject = function(context) {	
	var posX = this.x;
	var posY = this.y;
	var objWidth = this.width;
	var objHeigth = this.height;	
	var imageObj = new Image();					
	imageObj.onload = function(){		
		context.globalAlpha = 0.4;
					
		context.drawImage(imageObj, posX, posY, objWidth, objHeigth);
		flip();	
		
		context.globalAlpha = 1;
	}
	imageObj.src = this.imageUrl;
	
};

/* create memo object */
var webContentType4 = function(id,x, y, w, h,imageUrl,linkKind,destURI,destPageNumber) {			
	this.setup(id,x, y, w, h,imageUrl);	
	this.action = function() {					
		if(linkKind == "1"){
			window.open(destURI,"_blank","new window");	 
		}else if (linkKind == "0"){
			changePage(changePageNo(destPageNumber));	
		}
	};		
};
webContentType4.prototype = new WebGetContentTyp4Object();

/* ----------------------------------- End create webGetContentType4 -----------------------------*/

function imgBack_click() {
	/* set end log */
	SetEndLog(contentID);
	RegisterLog();
	
	window.onbeforeunload = null;	
	 
	if(ClientData.JumpQueue()){
		var oldDataBack = ClientData.JumpQueue();		
		if(oldDataBack.length > 0){			
			avwScreenMove(ScreenIds.ContentView);
			ClientData.IsJumpBack(true);
		}else{
			/*check back */
			if(ClientData.BookmarkScreen()){
				avwScreenMove(ClientData.BookmarkScreen());
			}else{
				window.history.back();
			}
		}
	}else{
		/*check back */
		if(ClientData.BookmarkScreen()){
				avwScreenMove(ClientData.BookmarkScreen());
		}else{
			window.history.back();
		}
	}
	
}

function imgHome_click(e) {
	e.preventDefault();
	
	/* set end log */
	SetEndLog(contentID);
	RegisterLog();
	
    //window.location.href = ScreenIds.Home;
    avwScreenMove(ScreenIds.Home);
}

$(function () {
    $("#imgBack").click(imgBack_click);
    $("#imgHome").click(imgHome_click);
    $("#btn_show_memo").click(showEditMemo);
    $("#btn_copy_memo").click(copyMemo);

    if (avwUserEnvObj.os == 'ipad') {

        if (ClientData.IsRefresh() != true) {
            $('body, html').hide();
            ClientData.IsRefresh(true);
            location.reload();
        }
        else {
            // Skip
        }

        //$(document).bind('touchmove', function (e) { e.preventDefault(); });
    }
});